#!/usr/bin/env perl
#
# Extracting texts from the AACT database of clinical trials.
#
# Usage: ./aact.pl -c COLUMN file1 [file2...] > output.txt
#
# Options:
# -c = name of the data column to be extracted
#
# The output contains the data of the given column in individual records (which contain
# newlines on paragraph boundaries), separated by double newlines. Empty records are skipped.
#
# The default AACT file for this kind of data is clinical_study.txt.
# Viable columns in this file are: BRIEF_SUMMARY, CRITERIA, DETAILED_DESCRIPTION
#

use strict;
use warnings;
use utf8;
use Getopt::Long;

# Copied from List::MoreUtils
sub firstidx (&@) {
    my $f = shift;
    foreach my $i ( 0 .. $#_ ) {
        local *_ = \$_[$i];
        return $i if $f->();
    }
    return -1;
}

binmode(STDOUT, ':utf8');

# initialize data structures
my @input_files;
my $col_name = 'BRIEF_SUMMARY';
my $USAGE = 'Usage: ./aact.pl [-c BRIEF_SUMMARY] clinical_study.txt > output.txt';

# handle options
GetOptions('column|c=s' => \$col_name) or die($USAGE);
die($USAGE) if (!@ARGV);
$col_name = uc $col_name;

# (main)
# process input files
foreach my $file (@ARGV){
    my $fh;

    if ($file =~ /\.gz$/){
        open($fh, 'gunzip -c "' . $file . '" |') or die('Cannot open:' . $file);
        binmode($fh, ':utf8');
    }
    else {
        open($fh, '<:utf8', $file) or die('Cannot open:' . $file);
    }

    my $header = <$fh>;
    my $col_no = firstidx { $_ eq $col_name } split(/\|/, $header);
    
    if ($col_no == -1){
        die("Column '$col_name' not found in file header!");
    } 
    
    my $buf = '';  # buffer for lines related to the same topic

    while (my $line = <$fh>){
        $buf .= $line;
        # end of record == CR-LF
        if ($line =~ /\r\n$/){
            my @cols = split(/\|/, $buf);
            my $output_data = $cols[$col_no];
            if ($output_data ne ''){            
                $output_data =~ s/\n(?=[^\n])//g;  # remove internal newlines
                $output_data =~ s/\n\n/\n/g;  # paragraphs by single newlines only
                $output_data =~ s/(\s) +/$1/g;  # clear spaces
                print  $output_data . "\n\n";
            }                      
            $buf = '';
        }
    }

}
