#!/usr/bin/perl

# Get parallel plain text from XML-annotated version of MuchMore.
#
# First un-tar the downloaded files:
# tar xzf springer_english_train_V4.2.tar.gz
# tar xzf springer_german_train_V4.2.tar.gz
#
# Usage: ./muchmore.pl path-to-uncompressed-XML-files \
# | iconv -f iso8859-1 -t utf8 > muchmore.tsv
#
# Note: sentence mapping in some files of MuchMore is broken, expect warnings
#       when running this script.
#
# Ales Tamchyna <tamchyna@ufal.mff.cuni.cz>

use 5.010;
use strict;
use warnings;
use Carp;
use File::Basename;

die "Usage: ./muchmore.pl path-to-uncompressed-XML-files > muchmore.tsv" if @ARGV < 1;

my $path = $ARGV[0];
my @files = glob "$path/*.ger.*";

for my $file (@files) {
  my $enfile = basename $file;
  $enfile =~ s/\.ger\./.eng./;
  next if ! -f "$path/$enfile"; # skip non-parallel docs
  open(my $de_hdl, $file);
  open(my $en_hdl, "$path/$enfile");
  my ($de_sents, $de_lines, $en_lines) = process_document($de_hdl);
  my ($en_sents) = process_document($en_hdl);
  close $de_hdl;
  close $en_hdl;

  while (@$de_lines) {
    my $de_segment = get_sentences($de_sents, shift @$de_lines);
    my $en_segment = get_sentences($en_sents, shift @$en_lines);
    if ($de_segment && $en_segment) {
      print "$de_segment\t$en_segment\n";
    }
  }
}

sub get_sentences {
  my ($sents, $which) = @_;
  my $out = "";
  while (my $id = shift @$which) {
    if ($id >= @$sents) {
      print STDERR "Index out of range: $id\n";
      return "";
    }
    $out .= $sents->[$id] . (@$which ? " " : "");
  }
  return $out;
}

sub process_document {
  my $hdl = shift;

  my @sents; # all sentences in doc
  my %mapping;
  my @tokens;
  while (<$hdl>) {
    chomp(my $line = $_);
    if ($line =~ m/<sentence id="s([0-9]+)".*corresp="([^"]*)"/) {
      my ($sent, $corresp_str) = ($1, $2);
      my @corresp;
      while ($corresp_str =~ m/s([0-9]+)/g) {
        push @corresp, $1 - 1;
      }
      $mapping{$sent - 1} = \@corresp if @corresp;
    } elsif ($line =~ m/<token [^<]*>(.*)<\/token>/) {
      push @tokens, $1;
    } elsif ($line =~ m/<\/sentence/) {
      push @sents, join(" ", @tokens);
      @tokens = ();
    }      
  }
  return (\@sents, shake_mapping(\%mapping));
}

# given a mapping, produce two lists of corresponding sentences
# e.g.: 1 -> 2,3 ; 2 -> 4 ; 3 -> 4
# returns ( [1], [2, 3] ) and ( [2, 3], [4] )
sub shake_mapping {
  my $mapping = shift;
  my @srclines;
  my @tgtlines;
  my @last = ( -1 );
  for my $id (sort { $a <=> $b } keys %$mapping) {
    my @aligned = @{ $mapping->{$id} };
    next if ! @aligned;
    if (@aligned ~~ @last) {
      push @{ $srclines[-1] }, $id;
    } else {
      push @srclines, [ $id ];    
      push @tgtlines, \@aligned;
    }
    @last = @aligned;
  }
  return (\@srclines, \@tgtlines);
}
