#!/usr/bin/env perl
#
# Extracting translation pairs from UMLS MRCONSO.RRF.* files (produces source-target pairs
# separated with tabs).
#
# Usage: ./umls-parallel.pl -s ENG -t GER [-p none|source|target|both] file1 [file2...] > output.tsv
#
# -s = source language ID
#      (language IDs are English abbreviations -- ENG, GER, CZE, FRE, SPA, POL, RUS, SWE, ITA, JPN...)
#
# -t = target language ID
#
# -p = output only "preferred" synonyms on the specified translation sides (default: none,
#      possible values: none, source, target, both)
#
# The default UMLS files for this kind of data are MRCONSO.RRF.*.gz.

use strict;
use warnings;
use utf8;
use Getopt::Long;

binmode(STDOUT, ':utf8'); 

# initialize data structures
my @input_files;
my $src_lang;
my $trg_lang;
my $preferred_only = 'none';
my $USAGE = 'Usage: ./umls-parallel.pl -s ENG -t GER [-c] [-p none|source|target|both] MRCONSO.RRF.*.gz > output.tsv';

# handle options
GetOptions('src_lang|s=s' => \$src_lang, 
        'trg_lang|t=s' => \$trg_lang, 
        'preferred_only|pref_only|p=s' => \$preferred_only) or die($USAGE);
die($USAGE) if (!$src_lang or !$trg_lang or !@ARGV or $preferred_only !~ /^[nstb]/);
$src_lang = uc $src_lang;
$trg_lang = uc $trg_lang;

# setup regexes for source and target so that they capture the preferred_only option
my $src_regex = $preferred_only =~ /^[sb]/ ? qr/^$src_lang\|P\|[^\|]*\|PF\|[^\|]*\|Y\|/ : qr/^$src_lang\|/;
my $trg_regex = $preferred_only =~ /^[tb]/ ? qr/^$trg_lang\|P\|[^\|]*\|PF\|[^\|]*\|Y\|/ : qr/^$trg_lang\|/;

# (main)
# process input files
foreach my $file (@ARGV){
    my $fh;

    if ($file =~ /\.gz$/){
        open($fh, 'gunzip -c "' . $file . '" |') or die('Cannot open:' . $file);
        binmode($fh, ':utf8');
    }
    else {
        open($fh, '<:utf8', $file) or die('Cannot open:' . $file);
    }

    my $last_concept = undef;
    my @buf = ();  # buffer for lines relating to the same concept
    while (my $line = <$fh>){
        # pre-parse the line (extract the concept ID)
        my ($concept, $rest) = split(/\|/, $line, 2);
        chomp($rest);
        # current line related to another concept -> handle all for the previous one
        if (defined($last_concept) and $concept ne $last_concept){
            handle_concept(@buf);
            @buf = ();
        }
        # add current line to buffer
        push @buf, $rest;    
        $last_concept = $concept;
    }

    # handle the last concept
    if (@buf){
        handle_concept(@buf);
    }
}


# print out texts for one concept that match the given language pair
sub handle_concept {
    # get lines matching to both languages and extract just the term text from them
    # remove duplicates by adding to a hash
    my %src_data_valid = map { (split /\|/)[13] => 1 } grep { $_ =~ /^$src_regex/ } @_;
    my @src_data = keys %src_data_valid;
    my %trg_data_valid = map { (split /\|/)[13] => 1 } grep { $_ =~ /^$trg_regex/ } @_; 
    my @trg_data = keys %trg_data_valid;

    # do nothing if there is no expression in one of the languages
    return if (!@src_data or !@trg_data);

    # print the output
    foreach my $src_expr (@src_data){
        foreach my $trg_expr (@trg_data){
            print "$src_expr\t$trg_expr\n";
        }
    }
}


